local VERSION = tonumber(('$Revision: 31573 $'):match('%d+'))

local PitBull = PitBull
local PitBull_Aura = PitBull:NewModule('Aura', 'AceEvent-2.0', 'AceConsole-2.0')
local self = PitBull_Aura
if PitBull.revision < VERSION then
	PitBull.version = 'r' .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ('$Date: 2007-04-01 18:38:40 -0400 (Sun, 01 Apr 2007) $'):match('%d%d%d%d%-%d%d%-%d%d')
end

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

local GameTooltip = GameTooltip
local UnitBuff = UnitBuff
local UnitDebuff = UnitDebuff
local math_floor = math.floor
local math_min = math.min

PitBull_Aura:RegisterPitBullChildFrames('auraFrame', 'auraFrame2')

local unitsShown = {}

function PitBull_Aura:OnInitialize()
	self.db = PitBull:AcquireDBNamespace('Aura')
	PitBull:RegisterDefaults('Aura', 'profile', {
		['**'] = {
			hidden = false,
			hiddenBuffs = false,
			hiddenDebuffs = false,
			split = false,
			filter = true,
		},
		target = {
			filter = false,
		}
	})
end

function PitBull_Aura:OnEnable(first)
	self:RegisterBucketEvent('UNIT_AURA', 0.05)
end

function PitBull_Aura:UNIT_AURA(units)
	for unit in pairs(units) do
		local frame = PitBull:GetUnitFrame(unit)
		if frame and unitsShown[unit] then
			self:UpdateAuras(unit, frame)
		end
	end
end

local function buffFrame_OnEnter(this)
	GameTooltip:SetOwner(this, 'ANCHOR_BOTTOMRIGHT')
	GameTooltip:SetUnitBuff(this.unit, this.id, this.raidFilter)
end
buffFrame_OnEnter = wrap(buffFrame_OnEnter, 'Aura/buffFrame_OnEnter')

local function debuffFrame_OnEnter(this)
	GameTooltip:SetOwner(this, 'ANCHOR_BOTTOMRIGHT')
	GameTooltip:SetUnitDebuff(this.unit, this.id, this.raidFilter)
end
debuffFrame_OnEnter = wrap(debuffFrame_OnEnter, 'Aura/debuffFrame_OnEnter')

local function buffFrame_OnLeave(this)
	GameTooltip:Hide()
end
buffFrame_OnLeave = wrap(buffFrame_OnLeave, 'Aura/buffFrame_OnLeave')
local debuffFrame_OnLeave = buffFrame_OnLeave

local maxbuffs, maxdebuffs, max_auras, num_shown_buffs = 32, 40

local function UpdateBuffs(unit, frame)
	local buffs = frame.auraFrame.buffs or new()
	local raidFilter = PitBull_Aura.db.profile[frame.group].filter and 1 or 0
	
	for i = 1, maxbuffs do
		local texture, applications
		
		_, _, texture, applications = UnitBuff(unit, i, raidFilter)
		if not texture then
			for j = #buffs, i, -1 do
				local v = buffs[j]
				buffs[j] = nil
				v.text = delFrame(v.text)
				v.icon = delFrame(v.icon)
				v.unit = nil
				v.id = nil
				v:SetScript('OnEnter', nil)
				v:SetScript('OnLeave', nil)
				delFrame(v)
			end
			break
		end
		
		local buffFrame = buffs[i] or newFrame('Button', frame.auraFrame)
		buffFrame:ClearAllPoints()
		buffFrame:SetWidth(16)
		buffFrame:SetHeight(16)
		local text = buffFrame.text or newFrame('FontString', buffFrame, 'OVERLAY')
		buffFrame.text = text
		text:SetFontObject(GameFontHighlightSmall)
		if applications <= 1 then
			text:SetText('')
		else
			text:SetText(applications)
		end
		text:SetPoint('BOTTOMRIGHT', 1, 0)
		local icon = buffFrame.icon or newFrame('Texture', buffFrame, 'BACKGROUND')
		buffFrame.icon = icon
		icon:SetTexture(texture)
		icon:SetAllPoints(buffFrame)
		buffFrame:SetScript('OnEnter', buffFrame_OnEnter)
		buffFrame:SetScript('OnLeave', buffFrame_OnLeave)
		buffFrame.id = i
		buffFrame.unit = unit
		buffFrame.raidFilter = raidFilter
		buffs[i] = buffFrame
	end
	local num_buffs = #buffs
	
	max_auras = math_floor(frame.auraFrame:GetWidth() / 16)
	num_shown_buffs = math_min(num_buffs, max_auras)
	
	for i = 1, num_shown_buffs do
		local buff = buffs[i]
		buff:Show()
		buff:ClearAllPoints()
		buff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', (i-1) * 16, 0)
	end
	for i = num_shown_buffs + 1, num_buffs do
		local buff = buffs[i]
		buff:Hide()
		buff:ClearAllPoints()
	end
	
	if num_buffs == 0 then
		buffs = del(buffs)
	end
	frame.auraFrame.buffs = buffs
end

local function UpdateDebuffs(unit, frame, buffs)
	local debuffs = frame.auraFrame.debuffs or new()
	local split, num_shown_debuffs = PitBull_Aura.db.profile[frame.group].split
	local raidFilter = PitBull_Aura.db.profile[frame.group].filter and 1 or 0
	
	for i = 1, maxdebuffs do
		local texture, applications, dispelType
		
		_, _, texture, applications, dispelType = UnitDebuff(unit, i, raidFilter)
		if not texture then
			for j = #debuffs, i, -1 do
				local v = debuffs[j]
				debuffs[j] = nil
				v.text = delFrame(v.text)
				v.icon = delFrame(v.icon)
				v.unit = nil
				v.id = nil
				v.raidFilter = nil
				v:SetScript('OnEnter', nil)
				v:SetScript('OnLeave', nil)
				delFrame(v)
			end
			break
		end
		
		local debuffFrame = debuffs[i] or newFrame('Button', split and frame.auraFrame2 or frame.auraFrame)
		debuffFrame:ClearAllPoints()
		debuffFrame:SetWidth(16)
		debuffFrame:SetHeight(16)
		local text = debuffFrame.text or newFrame('FontString', debuffFrame, 'OVERLAY')
		debuffFrame.text = text
		text:SetFontObject(GameFontHighlightSmall)
		if applications <= 1 then
			text:SetText('')
		else
			text:SetText(applications)
		end
		text:SetPoint('BOTTOMRIGHT', 1, 0)
		local icon = debuffFrame.icon or newFrame('Texture', debuffFrame, 'BACKGROUND')
		debuffFrame.icon = icon
		icon:SetTexture(texture)
		icon:SetAllPoints(debuffFrame)
		debuffFrame:SetScript('OnEnter', debuffFrame_OnEnter)
		debuffFrame:SetScript('OnLeave', debuffFrame_OnLeave)
		debuffFrame.id = i
		debuffFrame.unit = unit
		debuffFrame.raidFilter = raidFilter
		debuffs[i] = debuffFrame
	end
	local num_debuffs = #debuffs
	
	if split then
		local max_debuffs = math_floor(frame.auraFrame2:GetWidth() / 16)
		num_shown_debuffs = math_min(num_debuffs, max_debuffs)
	else
		max_auras = math_floor(frame.auraFrame:GetWidth() / 16)
		if not num_shown_buffs then
			num_shown_buffs = 0
		end
		num_shown_debuffs = math_min(num_debuffs, max_auras - num_shown_buffs)
	end
		for i = 1, num_shown_debuffs do
			local debuff = debuffs[i]
			debuff:Show()
			debuff:ClearAllPoints()
			if split then
				debuff:SetPoint('TOPLEFT', frame.auraFrame2, 'TOPLEFT', (i-1) * 16, 0)
			else
				if buffs then
					debuff:SetPoint('TOPRIGHT', frame.auraFrame, 'TOPRIGHT', (i-1) * -16, 0)
				else
					debuff:SetPoint('TOPLEFT', frame.auraFrame, 'TOPLEFT', (i-1) * 16, 0)
				end
			end
		end
		for i = num_shown_debuffs + 1, num_debuffs do
			local debuff = debuffs[i]
			debuff:Hide()
			debuff:ClearAllPoints()
		end
	
	
	if num_debuffs == 0 then
		debuffs = del(debuffs)
	end
	frame.auraFrame.debuffs = debuffs
end

function PitBull_Aura:UpdateAuras(unit, frame)
	if not unitsShown[unit] or self.db.profile[frame.group].hidden then
		return
	end
	local buffs
	if not self.db.profile[frame.group].hiddenBuffs then
		buffs = true
		UpdateBuffs(unit, frame)
	end
	if not self.db.profile[frame.group].hiddenDebuffs then
		UpdateDebuffs(unit, frame, buffs)
	end
end
PitBull_Aura.OnUpdateFrame = PitBull_Aura.UpdateAuras

function PitBull_Aura:OnPopulateUnitFrame(unit, frame)
	frame.auraFrame = newFrame('Frame')
	if self.db.profile.split then
		frame.auraFrame2 = newFrame('Frame')
	end
end

function PitBull_Aura:ClearAuras(unit, frame)
	local buffs = frame.auraFrame.buffs
	if buffs then
		for i = #buffs, 1, -1 do
			local v = buffs[i]
			buffs[i] = nil
			v.text = delFrame(v.text)
			v.icon = delFrame(v.icon)
			v.unit = nil
			v.id = nil
			v.raidFilter = nil
			v:SetScript('OnEnter', nil)
			v:SetScript('OnLeave', nil)
			delFrame(v)
		end
		frame.auraFrame.buffs = del(buffs)
	end

	local debuffs = frame.auraFrame.debuffs
	if debuffs then
		for i = #debuffs, 1, -1 do
			local v = debuffs[i]
			debuffs[i] = nil
			v.text = delFrame(v.text)
			v.icon = delFrame(v.icon)
			v.unit = nil
			v.id = nil
			v:SetScript('OnEnter', nil)
			v:SetScript('OnLeave', nil)
			delFrame(v)
		end
		frame.auraFrame.debuffs = del(debuffs)
	end
end

function PitBull_Aura:OnClearUnitFrame(unit, frame)
	if frame.auraFrame then
		self:ClearAuras(unit, frame)
	
		frame.auraFrame = delFrame(frame.auraFrame)
		if frame.auraFrame2 then
			frame.auraFrame2 = delFrame(frame.auraFrame2)
		end
	end
end

function PitBull_Aura:OnUnknownLayout(unit, frame, name)
	if name == 'auraFrame' then
		frame.auraFrame:SetHeight(16)
		frame.auraFrame:SetPoint('TOPLEFT', frame, 'BOTTOMLEFT')
		frame.auraFrame:SetPoint('TOPRIGHT', frame, 'BOTTOMRIGHT')
	elseif name == 'auraFrame2' then
		frame.auraFrame2:SetHeight(16)
		frame.auraFrame2:SetPoint('TOPLEFT', frame.auraFrame, 'BOTTOMLEFT')
		frame.auraFrame2:SetPoint('TOPRIGHT', frame.auraFrame, 'BOTTOMRIGHT')
	end
end
--
function PitBull_Aura:OnUpdateLayout(unit, frame)
	local old = not unitsShown[unit]
	local now = not frame.auraFrame or not frame.auraFrame:IsShown()
	if old == now then
		return
	end
	unitsShown[unit] = not now
	if now then
		self:ClearAuras(unit, frame)
	else
		self:UpdateAuras(unit, frame)
	end
end

local function getSplit(group)
	return PitBull_Aura.db.profile[group].split
end

local function setSplit(group, value)
	PitBull_Aura.db.profile[group].split = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(unit)
	end
end

local function getHidden(group)
	return PitBull_Aura.db.profile[group].hidden
end

function setHidden(group, value)
	PitBull_Aura.db.profile[group].hidden = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		if not value then
			PitBull_Aura:OnPopulateUnitFrame(unit, frame)
			PitBull_Aura:OnUpdateFrame(unit, frame)
		end	
		unitsShown[unit] = false
		PitBull:UpdateLayout(unit)
	end
end


local function getMultiHidden(table)
	local group, type = table.group, table.type
	return PitBull_Aura.db.profile[group]['hidden'..type]
end

local function setMultiHidden(table, value)
	local group, type = table.group, table.type
	PitBull_Aura.db.profile[group]['hidden'..type] = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnClearUnitFrame(unit, frame)
		PitBull_Aura:OnPopulateUnitFrame(unit, frame)
		PitBull_Aura:OnUpdateFrame(unit, frame)
		unitsShown[unit] = false
		PitBull:UpdateLayout(unit)
	end
end

local function getEnableFilter(group)
	return PitBull_Aura.db.profile[group].filter
end

local function setEnableFilter(group, value)
	PitBull_Aura.db.profile[group].filter = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Aura:OnUpdateFrame(unit, frame)
	end
end

PitBull_Aura:RegisterPitBullOptionsMethod(function(group)
	return {
		type = 'group',
		name = "Auras",
		desc = "Aura options",
		args = {
			Split = {
				type = 'toggle',
				name = 'Split',
				desc = 'Split buffs and debuffs',
				get = getSplit,
				set = setSplit,
				passValue = group,
			},
			Hide = {
				type = 'group',
				name = 'Hide',
				desc = 'Hide options',
				args = {
					All = {
						type = 'toggle',
						name = 'All',
						desc = 'Hide Buffs and Debuffs',
						get = getHidden,
						set = setHidden,
						passValue = group,
					},
					Buffs = {
						type = 'toggle',
						name = 'Buffs',
						desc = 'Hide buff bar',
						get = getMultiHidden,
						set = setMultiHidden,
						passValue = {['group'] = group, ['type'] = 'Buffs'}
					},
					Debuffs = {
						type = 'toggle',
						name = 'Debuffs',
						desc = 'Hide debuffs bar',
						get = getMultiHidden,
						set = setMultiHidden,
						passValue = {['group'] = group, ['type'] = 'Debuffs'}
					}
				}
			},
			filter = {
				type = 'group',
				name = "Filtering",
				desc = "Filter certain buffs and debuffs",
				args =  {
					enable = {
						type = 'toggle',
						name = "Enable filtering",
						desc = "Enable filtering",
						get = getEnableFilter,
						set = setEnableFilter,
						passValue = group,
					}
				}
			}
		}
	}
end)

wrap(PitBull_Aura, 'Aura')
