local VERSION = tonumber(("$Revision: 31513 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Class = PitBull:NewModule("Class")
local self = PitBull_Class
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-03-31 22:58:38 -0400 (Sat, 31 Mar 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localization placeholder
local L = setmetatable({}, {__index = function(self, key) self[key] = key; return key end })

local textStyles = {
	[L["Standard"]]    = "[classification][difficulty][level][white] [classcolor][smartclass][white] [smartrace]",
	[L["Hide"]]        = "",
	[L["Custom"]]      = "Custom",
	[L["Player Classes Only"]] = "[classification][difficulty][level][white] [classcolor][playerclass][white] [smartrace]",
	[L["Short"]]       = "[difficulty][level][plus][white] [smartrace]",
}

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

PitBull_Class:RegisterPitBullChildFrames('classText')

function PitBull_Class:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Class")
	PitBull:RegisterDefaults("Class", "profile", {
		['*'] = {
			textCustom = "[classification][difficulty][level][white] [classcolor][smartclass][white] [smartrace]",
			textStyle = L["Standard"],
		},
	})
end

function PitBull_Class:OnPopulateUnitFrame(unit, frame)
	local classText = newFrame("FontString", frame.overlay, "OVERLAY")
	frame.classText = classText
	classText:SetFontObject(GameFontHighlightSmall)
	classText:SetNonSpaceWrap(false)
	local code
	if self.db.profile[frame.group].textStyle == L["Custom"] then
		code = self.db.profile[frame.group].textCustom
	else
		code = textStyles[self.db.profile[frame.group].textStyle]
	end
	PitBull:RegisterWatchdogFontString(classText, unit, code)
end

function PitBull_Class:OnClearUnitFrame(unit, frame)
	frame.classText = delFrame(frame.classText)
end

function PitBull_Class:OnUnknownLayout(unit, frame, name)
	if frame.manaBar then
		frame.classText:SetPoint("LEFT", frame.manaBar, "LEFT", 3, 0)
		if frame.manaText then
			frame.classText:SetPoint("RIGHT", frame.manaText, "LEFT", -5, 0)
		end
	end
end

local function getTextStyle(group)
	return PitBull_Class.db.profile[group].textStyle
end
local function setTextStyle(group, value)
	PitBull_Class.db.profile[group].textStyle = value
	
	local code
	if value == L["Custom"] then
		code = PitBull_Class.db.profile[group].textCustom
	else
		code = textStyles[value]
	end
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:RegisterWatchdogFontString(frame.classText, unit, code)
	end
end
local function getTextCustom(group)
	return PitBull_Class.db.profile[group].textCustom
end
local function setTextCustom(group, value)
	PitBull_Class.db.profile[group].textCustom = value
	
	if PitBull_Class.db.profile[group].textStyle ~= L["Custom"] then
		return
	end
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:RegisterWatchdogFontString(frame.classText, unit, value)
	end
end

PitBull_Class:RegisterPitBullOptionsMethod(function(group)
	local textValidate = {}
	for k in pairs(textStyles) do
		textValidate[k] = k
	end
	return {
		type = 'group',
		name = L["Class Text"],
		desc = L["Class Text"],
		args = {
			style = {
				type = 'text',
				name = L["Style"],
				desc = L["Style"],
				validate = textValidate,
				get = getTextStyle,
				set = setTextStyle,
				passValue = group,
			},
			custom = {
				type = 'text',
				name = L["Custom"],
				desc = L["Custom"],
				get = getTextCustom,
				set = setTextCustom,
				usage = L["Watchdog tags"],
				passValue = group,
			}
		}
	}
end)

wrap(PitBull_Class, 'Class')
