local VERSION = tonumber(("$Revision: 31500 $"):match("%d+"))

local PitBull = PitBull
local PitBull_CombatText = PitBull:NewModule("CombatText", "AceEvent-2.0")
local self = PitBull_CombatText
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-03-31 20:06:03 -0400 (Sat, 31 Mar 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localization placeholder
local L = setmetatable({}, {__index = function(self, key) self[key] = key; return key end })

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

PitBull_CombatText:RegisterPitBullChildFrames('combatText')

local unitsUpdate = {}

function PitBull_CombatText:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("CombatText")
	PitBull:RegisterDefaults("CombatText", "profile", {
		groups = {
			['*'] = {},
		},
	})
end

function PitBull_CombatText:OnEnable(first)
	self:RegisterEvent("UNIT_COMBAT")
end


-- Combat Event stuff taken almost directly from agUF, credits to ag`
function PitBull_CombatText:UNIT_COMBAT( unit, event, flags, amount, type )
	local frame = PitBull:GetUnitFrame(unit)
	if frame and not self.db.profile.groups[frame.group].ignore then
		local fontHeight = 13
		local text = ""
		local r,g,b = 1,1,1
		if event == "IMMUNE" then
			fontHeight = fontHeight * .75
			text = CombatFeedbackText["IMMUNE"]
		elseif event == "WOUND" then
			if amount ~=0 then
				if flags == "CRITICAL" or flags == "CRUSHING" then
					fontHeight = fontHeight * 1.5
				elseif flags == "GLANCING" then
					fontHeight = fontHeight * .75
				end
				if type > 0 then
					r = 1
					g = 1
					b = 0
				end
				if UnitInParty( unit ) or UnitInRaid( unit ) then
					r = 1
					g = 0
					b = 0
				end
				text = "-"..amount
			elseif flags == "ABSORB" or flags == "BLOCK" or flags == "RESIST" then
				fontHeight = fontHeight * .75
				text = CombatFeedbackText[event]
			else
				text = CombatFeedbackText[event]
			end
		elseif event == "BLOCK" then
			fontHeight = fontHeight * .75
			text = CombatFeedbackText[event]	 		
		elseif event == "HEAL" then
			text = "+"..amount
			r = 0
			g = 1
			b = 0
			if ( flags == "CRITICAL" ) then
				fontHeight = fontHeight * 1.3
			end			
		elseif event == "ENERGIZE" then
			text = amount
			r = 0.41
			g = 0.8
			b = 0.94
			if flags == "CRITICAL" then
				fontHeight = fontHeight * 1.3
			end
		else
			text = CombatFeedbackText[event]
		end
		
		unitsUpdate[unit] = GetTime()
		local font = frame.combatText:GetFont()
		frame.combatText:SetFont(font,fontHeight,"OUTLINE")
		frame.combatText:SetText(text)
		frame.combatText:SetTextColor(r, g, b)
		frame.combatText:SetAlpha(0)
		frame.combatText:Show()
		self:ScheduleRepeatingEvent("PitBull_CombatTextSchedule", self.updateCombatText, 0.05, self)
	end
end

function PitBull_CombatText:updateCombatText()
	local found, elapsedTime, alpha
	local maxalpha = 0.6
	local frame
	local now = GetTime()
	for unit, time in pairs( unitsUpdate ) do
		frame = PitBull:GetUnitFrame(unit)
		if frame then
			found = true
			elapsedTime = now - time
			if ( elapsedTime < COMBATFEEDBACK_FADEINTIME ) then
				alpha = maxalpha*(elapsedTime / COMBATFEEDBACK_FADEINTIME)
				frame.combatText:SetAlpha(alpha)
			elseif ( elapsedTime < (COMBATFEEDBACK_FADEINTIME + COMBATFEEDBACK_HOLDTIME) ) then
				frame.combatText:SetAlpha(maxalpha)
			elseif ( elapsedTime < (COMBATFEEDBACK_FADEINTIME + COMBATFEEDBACK_HOLDTIME + COMBATFEEDBACK_FADEOUTTIME) ) then
				alpha = maxalpha - maxalpha*((elapsedTime - COMBATFEEDBACK_HOLDTIME - COMBATFEEDBACK_FADEINTIME) / COMBATFEEDBACK_FADEOUTTIME)
				frame.combatText:SetAlpha(alpha)
			else
				frame.combatText:Hide()
				unitsUpdate[unit] = nil
			end		
		else
			unitsUpdate[unit] = nil
		end
		
		found = true
		
	end
	if not found then
		self:CancelScheduledEvent("PitBull_CombatTextSchedule")
	end
end

function PitBull_CombatText:OnPopulateUnitFrame(unit, frame)
	if not self.db.profile.groups[frame.group].ignore then
		local combatText = newFrame("FontString", frame.overlay, "OVERLAY")
		frame.combatText = combatText
		combatText:SetFontObject(GameFontNormal)
		combatText:SetNonSpaceWrap(false)
		combatText:Hide()
	end
end

function PitBull_CombatText:OnClearUnitFrame(unit, frame)
	if frame.combatText then
		frame.combatText = delFrame(frame.combatText)
	end
end

function PitBull_CombatText:OnUnknownLayout(unit, frame, name)
	if frame.combatText then
		frame.combatText:SetPoint("CENTER", frame, "CENTER")
	end
end

local function getIgnored(group)
	return PitBull_CombatText.db.profile.groups[group].ignore
end
local function setIgnored(group, value)
	PitBull_CombatText.db.profile.groups[group].ignore = value
	if value then
		for unit,frame in PitBull:IterateUnitFramesByGroup(group) do
			if frame.combatText then
				PitBull_CombatText:OnClearUnitFrame(unit, frame)
				unitsUpdate[unit] = nil
			end
		end
	else
		for unit,frame in PitBull:IterateUnitFramesByGroup(group) do
			if not frame.combatText then
				PitBull_CombatText:OnPopulateUnitFrame(unit, frame)
				PitBull:UpdateLayout(unit)
			end
		end
	end
end
PitBull_CombatText:RegisterPitBullOptionsMethod(function(group)
	return {
		name = L["Combat Text"],
		desc = L["Combat Text"],
		type = 'group',
		args = {
			ignore = {
				type = 'toggle',
				name = L["Disable Combat Text"],
				desc = L["Disable Combat Text"],
				get = getIgnored,
				set = setIgnored,
				passValue = group,
			},
		}
	}
end)

wrap(PitBull_CombatText, 'CombatText')