local VERSION = tonumber(("$Revision: 31602 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Health = PitBull:NewModule("Health", "AceEvent-2.0", "AceConsole-2.0")
local self = PitBull_Health
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-02 01:51:56 -0400 (Mon, 02 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitClass = UnitClass

PitBull_Health:RegisterPitBullChildFrames('healthBar', 'healthText')

local unitsShown = {}

local textStyles = {
	["Absolute"]       = "[statusgone:smarthp]",
	["Absolute Short"] = "[statusgone:smarthp:short]",
	["Difference"]     = "[statusgone:smartmissinghp]",
	["Percent"]        = "[statusgone:percenthp:percent]",
	["Hide"]           = "",
	["Custom"]         = "Custom",
	["Smart"]          = "|cffff7f7f[missinghp:hidezero:short]|r",
}

function PitBull_Health:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Health")
	PitBull:RegisterDefaults("Health", "profile", {
		['*'] = {
			classColor = true,
			classColorPvP = false,
			hostilityColor = true,
			deficit = false,
			reverse = false,
			textCustom = "[curhp]/[maxhp]",
			textStyle = "Absolute",
		}
	})
end

function PitBull_Health:OnEnable(first)
	self:RegisterBucketEvent({"UNIT_HEALTH", "UNIT_MAXHEALTH" }, 0.05, "UNIT_HEALTH")
	self:RegisterBucketEvent("UNIT_FACTION", 0.05, "UNIT_FACTION")
end

function PitBull_Health:UNIT_HEALTH(units)
	for unit in pairs(units) do
		local frame = PitBull:GetUnitFrame(unit)
		if frame and unitsShown[unit] then
			self:UpdateHealth(unit, frame)
			self:UpdateHealthbarColor(unit, frame)
		end
	end
end

function PitBull_Health:UNIT_FACTION(units)
	for unit in pairs(units) do
		local frame = PitBull:GetUnitFrame(unit)
		if frame and unitsShown[unit] then
			self:UpdateHealthbarColor(unit, frame)
		end
	end
end

function PitBull_Health:UpdateHealth(unit, frame)
	local db = self.db.profile[frame.group]
	
	local max = UnitHealthMax(unit)
	if max > 0 then
		local value = UnitHealth(unit)
		local perc = value / max
		if db.deficit ~= db.reverse then
			perc = 1 - perc
		end
		frame.healthBar:SetValue(perc)
	end
end

local _,class = UnitClass("player")
function PitBull_Health:UpdateHealthbarColor(unit, frame)
	local db = self.db.profile[frame.group]
	
	local r, g, b
	if UnitIsDeadOrGhost(unit) then
		r, g, b = unpack(PitBull.colorConstants.dead)
	elseif not UnitIsConnected(unit) then
		r, g, b = unpack(PitBull.colorConstants.disconnected)
	elseif UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
		r, g, b = unpack(PitBull.colorConstants.tapped)
	elseif UnitIsPlayer(unit) then
		if db.classColor and (db.classColorPvP or UnitIsFriend("player", unit)) then
			local _,class = UnitClass(unit)
			r, g, b = unpack(PitBull.colorConstants[class or 'WARRIOR'])
		elseif db.hostilityColor then
			if UnitCanAttack(unit, "player") then
				-- they can attack me
				if UnitCanAttack("player", unit) then
					-- and I can attack them
					r, g, b = unpack(PitBull.colorConstants.hostile)
				else
					-- but I can't attack them
					r, g, b = unpack(PitBull.colorConstants.civilian)
				end
			elseif UnitCanAttack("player", unit) then
				-- they can't attack me, but I can attack them
				r, g, b = unpack(PitBull.colorConstants.neutral)
			elseif UnitIsFriend("player", unit) then
				-- on my team
				r, g, b = unpack(PitBull.colorConstants.friendly)
			else
				-- either enemy or friend, no violence
				r, g, b = unpack(PitBull.colorConstants.civilian)
			end
		end
	elseif UnitIsUnit(unit, "pet") then
		if class == "HUNTER" then
			local happy = GetPetHappiness()
			if happy == 3 then
				r, g, b = unpack(PitBull.colorConstants.petHappy)
			elseif happy == 2 then
				r, g, b = unpack(PitBull.colorConstants.petNeutral)
			elseif happy == 1 then
				r, g, b = unpack(PitBull.colorConstants.petAngry)
			end
		end
	elseif db.hostilityColor then
		local reaction = UnitReaction(unit, "player")
		if reaction then
			if reaction >= 5 then
				r, g, b = unpack(PitBull.colorConstants.friendly)
			elseif reaction == 4 then
				r, g, b = unpack(PitBull.colorConstants.neutral)
			else
				r, g, b = unpack(PitBull.colorConstants.hostile)
			end
		else
			r, g, b = unpack(PitBull.colorConstants.unknown)
		end
	end
	if not r then
		local perc = UnitHealth(unit) / UnitHealthMax(unit)
		b = 0
		if perc > 0.5 then
			g = 1
			r = (1 - perc) * 2
		else
			r = 1
			g = perc * 2
		end
	end
	if db.reverse then
		frame.healthBar:SetStatusBarColor((r+0.2)/3, (g+0.2)/3, (b+0.2)/3, 1)
		frame.healthBar.bg:SetVertexColor(r, g, b, 1)
	else
		frame.healthBar:SetStatusBarColor(r, g, b, 1)
		frame.healthBar.bg:SetVertexColor((r+0.2)/3, (g+0.2)/3, (b+0.2)/3, 1)
	end
end

function PitBull_Health:OnUpdateFrame(unit, frame)
	if unitsShown[unit] then
		self:UpdateHealth(unit, frame)
		self:UpdateHealthbarColor(unit, frame)
	end
end

function PitBull_Health:OnPopulateUnitFrame(unit, frame)
	local texture = PitBull:GetStatusBarTexture()
	
	local healthBar = newFrame("StatusBar", frame)
	frame.healthBar = healthBar
	healthBar:SetMinMaxValues(0, 1)
	healthBar:SetValue(1)
	healthBar:SetStatusBarTexture(texture)
	healthBar:SetStatusBarColor(0, 1, 0, 1)

	local healthBarBG = newFrame("Texture", healthBar, "BACKGROUND")
	frame.healthBar.bg = healthBarBG
	healthBarBG:SetTexture(texture)
	healthBarBG:SetVertexColor(0, 0.5, 0, 1)
	healthBarBG:ClearAllPoints()
	healthBarBG:SetAllPoints(healthBar)

	local healthText = newFrame("FontString", frame.overlay, "OVERLAY")
	frame.healthText = healthText
	healthText:SetFontObject(GameFontHighlightSmall)
	healthText:SetNonSpaceWrap(false)
	local code
	if self.db.profile[frame.group].textStyle == "Custom" then
		code = self.db.profile[frame.group].textCustom
	else
		code = textStyles[self.db.profile[frame.group].textStyle]
	end
	PitBull:RegisterWatchdogFontString(healthText, unit, code)
end

function PitBull_Health:OnClearUnitFrame(unit, frame)
	frame.healthBar.bg = delFrame(frame.healthBar.bg)
	frame.healthBar = delFrame(frame.healthBar)
	frame.healthText = delFrame(frame.healthText)
end

function PitBull_Health:OnUpdateStatusBarTexture(texture)
	for _,frame in PitBull:IterateUnitFrames() do
		frame.healthBar:SetStatusBarTexture(texture)
		frame.healthBar.bg:SetTexture(texture)
	end
end

function PitBull_Health:OnUnknownLayout(unit, frame, name)
	if name == 'healthBar' then
		frame.healthBar:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5)
		frame.healthBar:SetPoint("BOTTOMRIGHT", frame, "RIGHT", -5, 0)
	elseif name == 'healthText' then
		frame.healthText:SetPoint("RIGHT", frame.healthBar, "RIGHT", -3, 0)
	end
end

function PitBull_Health:OnUpdateLayout(unit, frame)
	local old = not unitsShown[unit]
	local now = not frame.healthBar:IsShown()
	if old == now then
		return
	end
	unitsShown[unit] = not now
	if not now then
		self:OnUpdateFrame(unit, frame)
	end
end

local function getClassColor(group)
	return PitBull_Health.db.profile[group].classColor
end
local function setClassColor(group, value)
	PitBull_Health.db.profile[group].classColor = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end
local function getClassColorPvP(group)
	return PitBull_Health.db.profile[group].classColorPvP
end
local function setClassColorPvP(group, value)
	PitBull_Health.db.profile[group].classColorPvP = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end
local function getHostilityColor(group)
	return PitBull_Health.db.profile[group].hostilityColor
end
local function setHostilityColor(group, value)
	PitBull_Health.db.profile[group].hostilityColor = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end
local function getDeficit(group)
	return PitBull_Health.db.profile[group].deficit
end
local function setDeficit(group, value)
	PitBull_Health.db.profile[group].deficit = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
	end
end
local function getReverse(group)
	return PitBull_Health.db.profile[group].reverse
end
local function setReverse(group, value)
	PitBull_Health.db.profile[group].reverse = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end
local function getTextStyle(group)
	return PitBull_Health.db.profile[group].textStyle
end
local function setTextStyle(group, value)
	PitBull_Health.db.profile[group].textStyle = value
	
	local code
	if value == "Custom" then
		code = PitBull_Health.db.profile[group].textCustom
	else
		code = textStyles[value]
	end
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:RegisterWatchdogFontString(frame.healthText, unit, code)
	end
end
local function getTextCustom(group)
	return PitBull_Health.db.profile[group].textCustom:gsub("|", "||")
end
local function setTextCustom(group, value)
	PitBull_Health.db.profile[group].textCustom = value:gsub("||", "|")
	
	if PitBull_Health.db.profile[group].textStyle ~= "Custom" then
		return
	end
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:RegisterWatchdogFontString(frame.healthText, unit, value)
	end
end
PitBull_Health:RegisterPitBullOptionsMethod(function(group)
	local hostility = nil
	if group == "focus" or group:find("target$") then
		hostility = true
	end
	local textValidate = {}
	for k in pairs(textStyles) do
		textValidate[k] = k
	end
	return {
		name = "Health",
		desc = "Health",
		type = 'group',
		args = {
			text = {
				type = 'group',
				name = "Text",
				desc = "Text",
				args = {
					style = {
						type = 'text',
						name = "Style",
						desc = "Style",
						validate = textValidate,
						get = getTextStyle,
						set = setTextStyle,
						passValue = group,
					},
					custom = {
						type = 'text',
						name = "Custom",
						desc = "Custom",
						get = getTextCustom,
						set = setTextCustom,
						usage = "Watchdog tags",
						passValue = group,
					}
				}
			},
			classColor = {
				type = 'toggle',
				name = "Color health bar by class",
				desc = "Color health bar by class",
				get = getClassColor,
				set = setClassColor,
				passValue = group,
			},
			classColorPvP = hostility and {
				type = 'toggle',
				name = "Color health by class for PvP enemies",
				desc = "Color PvP enemies health bars by class",
				get = getClassColorPvP,
				set = setClassColorPvP,
				passValue = group,
			},
			hostilityColor = hostility and {
				type = 'toggle',
				name = "Color health bar by hostility",
				desc = "Color health bar by hostility. Note: this takes precedence after color by class.",
				get = getHostilityColor,
				set = setHostilityColor,
				passValue = group,
			},
			deficit = {
				type = 'toggle',
				name = "Show health bar as a deficit",
				desc = "Show health bar as a deficit",
				get = getDeficit,
				set = setDeficit,
				passValue = group,
			},
			reverse = {
				type = 'toggle',
				name = "Reverse health bar direction",
				desc = "Reverse health bar direction",
				get = getReverse,
				set = setReverse,
				passValue = group,
			}
		}
	}
end)

wrap(PitBull_Health, 'Health')
