local VERSION = tonumber(("$Revision: 31472 $"):match("%d+"))

local PitBull = PitBull
local PitBull_HideBlizzard = PitBull:NewModule("HideBlizzard", "AceEvent-2.0")
local self = PitBull_HideBlizzard
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-03-31 06:48:15 -0400 (Sat, 31 Mar 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

local UnitIsUnit = UnitIsUnit

function PitBull_HideBlizzard:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("HideBlizzard")
	PitBull:RegisterDefaults("HideBlizzard", "profile", {
		hidePlayerFrame = true,
		hidePartyFrame = true,
		hideTargetFrame = true,
		hideCastBar = true,
	})

	PitBull.options.args.HideBlizzard = {
		type = 'group',
		name = "Hide Blizzard frames",
		desc = "Hide Blizzard frames",
		args = {
			player = {
				type = 'toggle',
				name = "Hide Player frame",
				desc = "Hide Player frame",
				get = function()
					return self.db.profile.hidePlayerFrame
				end,
				set = function(value)
					self.db.profile.hidePlayerFrame = value
					self:UpdateBlizzardFrames()
				end
			},
			party = {
				type = 'toggle',
				name = "Hide Party frames",
				desc = "Hide Party frames",
				get = function()
					return self.db.profile.hidePartyFrame
				end,
				set = function(value)
					self.db.profile.hidePartyFrame = value
					self:UpdateBlizzardFrames()
				end
			},
			target = {
				type = 'toggle',
				name = "Hide Target frame",
				desc = "Hide Target frame",
				get = function()
					return self.db.profile.hideTargetFrame
				end,
				set = function(value)
					self.db.profile.hideTargetFrame = value
					self:UpdateBlizzardFrames()
				end
			},
			castBar = {
				type = 'toggle',
				name = "Hide Cast bar",
				desc = "Hide Cast bar",
				get = function()
					return self.db.profile.hideCastBar
				end,
				set = function(value)
					self.db.profile.hideCastBar = value
					self:UpdateBlizzardFrames()
				end
			},
		},
		hidden = function()
			return not PitBull:IsModuleActive("HideBlizzard")
		end,
	}
end

function PitBull_HideBlizzard:OnEnable(first)
	self:UpdateBlizzardFrames()
end

function PitBull_HideBlizzard:UpdateBlizzardFrames()
	if self.db.profile.hidePlayerFrame then
		self:HideBlizzardPlayerFrame()
	else
		self:ShowBlizzardPlayerFrame()
	end
	if self.db.profile.hidePartyFrame then
		self:HideBlizzardPartyFrames()
	else
		self:ShowBlizzardPartyFrames()
	end
	if self.db.profile.hideTargetFrame then
		self:HideBlizzardTargetFrame()
	else
		self:ShowBlizzardTargetFrame()
	end
	if self.db.profile.hideCastBar then
		self:HideBlizzardCastingBar()
	else	
		self:ShowBlizzardCastingBar()
	end
end

function PitBull_HideBlizzard:HideBlizzardPlayerFrame()
	PlayerFrame:UnregisterAllEvents()
	PlayerFrameHealthBar:UnregisterAllEvents()
	PlayerFrameManaBar:UnregisterAllEvents()
	PlayerFrame:Hide()
end

function PitBull_HideBlizzard:ShowBlizzardPlayerFrame()
	PlayerFrame:RegisterEvent("UNIT_LEVEL")
	PlayerFrame:RegisterEvent("UNIT_COMBAT")
	PlayerFrame:RegisterEvent("UNIT_SPELLMISS")
	PlayerFrame:RegisterEvent("UNIT_PVP_UPDATE")
	PlayerFrame:RegisterEvent("UNIT_MAXMANA")
	PlayerFrame:RegisterEvent("PLAYER_ENTER_COMBAT")
	PlayerFrame:RegisterEvent("PLAYER_LEAVE_COMBAT")
	PlayerFrame:RegisterEvent("PLAYER_UPDATE_RESTING")
	PlayerFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	PlayerFrame:RegisterEvent("PARTY_LEADER_CHANGED")
	PlayerFrame:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
	PlayerFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
	PlayerFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	PlayerFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
	PlayerFrameHealthBar:RegisterEvent("UNIT_HEALTH")
	PlayerFrameHealthBar:RegisterEvent("UNIT_MAXHEALTH")
	PlayerFrameManaBar:RegisterEvent("UNIT_MANA")
	PlayerFrameManaBar:RegisterEvent("UNIT_RAGE")
	PlayerFrameManaBar:RegisterEvent("UNIT_FOCUS")
	PlayerFrameManaBar:RegisterEvent("UNIT_ENERGY")
	PlayerFrameManaBar:RegisterEvent("UNIT_HAPPINESS")
	PlayerFrameManaBar:RegisterEvent("UNIT_MAXMANA")
	PlayerFrameManaBar:RegisterEvent("UNIT_MAXRAGE")
	PlayerFrameManaBar:RegisterEvent("UNIT_MAXFOCUS")
	PlayerFrameManaBar:RegisterEvent("UNIT_MAXENERGY")
	PlayerFrameManaBar:RegisterEvent("UNIT_MAXHAPPINESS")
	PlayerFrameManaBar:RegisterEvent("UNIT_DISPLAYPOWER")
	PlayerFrame:RegisterEvent("UNIT_NAME_UPDATE")
	PlayerFrame:RegisterEvent("UNIT_PORTRAIT_UPDATE")
	PlayerFrame:RegisterEvent("UNIT_DISPLAYPOWER")
	PlayerFrame:Show()
end

function PitBull_HideBlizzard:HideBlizzardPartyFrames()
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame:UnregisterAllEvents()
		frame:Hide()
		frame.Show = function() end
	end
end

function PitBull_HideBlizzard:ShowBlizzardPartyFrames()
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame.Show = nil
		frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
		frame:RegisterEvent("PARTY_LEADER_CHANGED")
		frame:RegisterEvent("PARTY_MEMBER_ENABLE")
		frame:RegisterEvent("PARTY_MEMBER_DISABLE")
		frame:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
		frame:RegisterEvent("UNIT_PVP_UPDATE")
		frame:RegisterEvent("UNIT_AURA")
		frame:RegisterEvent("UNIT_PET")
		frame:RegisterEvent("VARIABLES_LOADED")
		frame:RegisterEvent("UNIT_NAME_UPDATE")
		frame:RegisterEvent("UNIT_PORTRAIT_UPDATE")
		frame:RegisterEvent("UNIT_DISPLAYPOWER")

		UnitFrame_OnEvent("PARTY_MEMBERS_CHANGED")
		
		this = frame
		PartyMemberFrame_UpdateMember()
	end
end

function PitBull_HideBlizzard:HideBlizzardTargetFrame()
	TargetFrame:UnregisterAllEvents()
	TargetFrame:Hide()

	ComboFrame:UnregisterAllEvents()
end

function PitBull_HideBlizzard:ShowBlizzardTargetFrame()
	TargetFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	TargetFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
	TargetFrame:RegisterEvent("UNIT_HEALTH")
	TargetFrame:RegisterEvent("UNIT_LEVEL")
	TargetFrame:RegisterEvent("UNIT_FACTION")
	TargetFrame:RegisterEvent("UNIT_CLASSIFICATION_CHANGED")
	TargetFrame:RegisterEvent("UNIT_AURA")
	TargetFrame:RegisterEvent("PLAYER_FLAGS_CHANGED")
	TargetFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	if UnitExists("target") then
		TargetFrame:Show()
	end

	ComboFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
	ComboFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	ComboFrame:RegisterEvent("PLAYER_COMBO_POINTS")
end

function PitBull_HideBlizzard:HideBlizzardCastingBar()
	CastingBarFrame:UnregisterAllEvents()
end

function PitBull_HideBlizzard:ShowBlizzardCastingBar()
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
end

wrap(PitBull_HideBlizzard, 'HideBlizzard')

