local VERSION = tonumber(("$Revision: 31606 $"):match("%d+"))

local PitBull = PitBull
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-02 03:28:33 -0400 (Mon, 02 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local ABF_db = PitBull:AcquireDBNamespace("Layout-ABF")
PitBull:RegisterDefaults("Layout-ABF", 'profile', {
	['**'] = {
		portraitSide = "left"
	},
	target = {
		portraitSide = "right"
	}
})

local function portraitSide_get(group)
	return ABF_db.profile[group].portraitSide
end
local function portraitSide_set(group, value)
	ABF_db.profile[group].portraitSide = value
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:UpdateLayout(unit)
	end
end
PitBull:RegisterLayout("ABF", {
	healthBar = function(unit, frame)
		if frame.portrait then
			if ABF_db.profile[frame.group].portraitSide == "right" then
				frame.healthBar:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5)
				if not frame.manaBar then
					frame.healthBar:SetPoint("BOTTOMRIGHT", frame.portrait, "BOTTOMLEFT", -5, 0)
				else
					frame.healthBar:SetPoint("BOTTOMRIGHT", frame.portrait, "LEFT", -5, 0)
				end
			else
				frame.healthBar:SetPoint("TOPLEFT", frame.portrait, "TOPRIGHT", 5, 0)
				if not frame.manaBar then
					frame.healthBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, 5)
				else
					frame.healthBar:SetPoint("BOTTOMRIGHT", frame, "RIGHT", -5, 0)
				end
			end
		else
			frame.healthBar:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5)
			if not frame.manaBar then
				frame.healthBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, 5)
			else
				frame.healthBar:SetPoint("BOTTOMRIGHT", frame, "RIGHT", -5, 0)
			end
		end
	end,
	manaBar = function(unit, frame)
		if frame.portrait then
			if ABF_db.profile[frame.group].portraitSide == "right" then
				frame.manaBar:SetPoint("TOPLEFT", frame, "LEFT", 5, 0)
				frame.manaBar:SetPoint("BOTTOMRIGHT", frame.portrait, "BOTTOMLEFT", -5, 0)
			else
				frame.manaBar:SetPoint("TOPLEFT", frame.portrait, "RIGHT", 5, 0)
				frame.manaBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, 5)
			end
		else
			frame.manaBar:SetPoint("TOPLEFT", frame, "LEFT", 5, 0)
			frame.manaBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, 5)
		end
	end,
	healthText = function(unit, frame)
		if frame.healthBar then
			frame.healthText:SetPoint("RIGHT", frame.healthBar, "RIGHT", -3, 0)
		end
	end,
	manaText = function(unit, frame)
		if frame.manaBar then
			frame.manaText:SetPoint("RIGHT", frame.manaBar, "RIGHT", -3, 0)
		end	
	end,
	nameText = function(unit, frame)
		if frame.healthBar then
			frame.nameText:SetPoint("LEFT", frame.healthBar, "LEFT", 3, 0)
			if frame.healthText then
				frame.nameText:SetPoint("RIGHT", frame.healthText, "LEFT", -5, 0)
			end
		end
	end,
	classText = function(unit, frame)
		if frame.manaBar then
			frame.classText:SetPoint("LEFT", frame.manaBar, "LEFT", 3, 0)
			if frame.manaText then
				frame.classText:SetPoint("RIGHT", frame.manaText, "LEFT", -5, 0)
			end
		end
	end,
	auraFrame = function(unit, frame)
		frame.auraFrame:SetHeight(16)
		frame.auraFrame:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 5, 0)
		frame.auraFrame:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT", -5, 0)
	end,
	portrait = function(unit, frame)
		if ABF_db.profile[frame.group].portraitSide ~= "right" then
			frame.portrait:SetPoint("LEFT", frame, "LEFT", 5, 0)
		else
			frame.portrait:SetPoint("RIGHT", frame, "RIGHT", -5, 0)
		end
	end,
	portrait_options = function(group)
		return {
			name = "Portrait side",
			desc = "Portrait side",
			type = 'text',
			validate = {
				left = "Left",
				right = "Right",
			},
			get = portraitSide_get,
			set = portraitSide_set,
			passValue = group
		}
	end,
})
