local VERSION = tonumber(("$Revision: 31272 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Leader = PitBull:NewModule("Leader", "AceEvent-2.0")
local self = PitBull_Leader
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-03-29 13:31:12 -0400 (Thu, 29 Mar 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

PitBull_Leader:RegisterPitBullChildFrames('leaderIcon')

function PitBull_Leader:OnEnable()
	self:RegisterEvent("PARTY_LEADER_CHANGED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "PARTY_LEADER_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "PARTY_LEADER_CHANGED")
end

function PitBull_Leader:Update(unit, frame)
	local has = true
	if unit == "player" then
		if not IsPartyLeader() then
			has = false
		end
	else
		local raid = unit:match("^raid(%d%d?)$")
		if raid then
			local _, rank = GetRaidRosterInfo(tonumber(raid))
			if rank ~= 2 then
				has = false
			end
		else
			local party = unit:match("^party(%d)$")
			if party then
				if GetPartyLeaderIndex() ~= tonumber(party) then
					has = false
				end
			else
				has = false
			end
		end
	end
	
	if has then
		if not frame.leaderIcon then
			local leaderIcon = newFrame("Texture", frame.overlay, "ARTWORK")
			frame.leaderIcon = leaderIcon
			leaderIcon:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
			
			PitBull:UpdateLayout(unit)
		end
	else
		if frame.leaderIcon then
			frame.leaderIcon = delFrame(frame.leaderIcon)
			
			PitBull:UpdateLayout(unit)
		end
	end
end

function PitBull_Leader:PARTY_LEADER_CHANGED()
	for unit, frame in PitBull:IterateNonWackyUnitFrames() do
		self:Update(unit, frame)
	end
end

function PitBull_Leader:OnUpdateFrame(unit, frame)
	self:Update(unit, frame)
end

function PitBull_Leader:OnClearUnitFrame(unit, frame)
	if frame.leaderIcon then
		frame.leaderIcon = delFrame(frame.leaderIcon)
	end
end

function PitBull_Leader:OnUnknownLayout(unit, frame)
	frame.leaderIcon:SetPoint("TOPLEFT", frame, "TOPLEFT", -2, 2)
	frame.leaderIcon:SetWidth(12)
	frame.leaderIcon:SetHeight(12)
end

wrap(PitBull_Leader, 'Leader')
