local VERSION = tonumber(("$Revision: 31574 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Name = PitBull:NewModule("Name")
local self = PitBull_Name
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-01 18:45:28 -0400 (Sun, 01 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localization placeholder
local L = setmetatable({}, {__index = function(self, key) self[key] = key; return key end })

local textStyles = {
	[L["Standard"]]    = "[name]",
	[L["Hide"]]        = "",
	[L["Custom"]]      = "Custom",
	[L["Class Colored"]] = "[classcolor][name]",
	[L["Long"]]       = "[level] [classcolor][name]",
}

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

PitBull_Name:RegisterPitBullChildFrames('nameText')

function PitBull_Name:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Name")
	PitBull:RegisterDefaults("Name", "profile", {
		['*'] = {
			textCustom = "[name]",
			textStyle = L["Standard"],
		},
	})
end

function PitBull_Name:OnPopulateUnitFrame(unit, frame)
	local nameText = newFrame("FontString", frame.overlay, "OVERLAY")
	frame.nameText = nameText
	nameText:SetFontObject(GameFontHighlightSmall)
	nameText:SetNonSpaceWrap(false)
	if self.db.profile[frame.group].textStyle == L["Custom"] then
		code = self.db.profile[frame.group].textCustom
	else
		code = textStyles[self.db.profile[frame.group].textStyle]
	end
	PitBull:RegisterWatchdogFontString(nameText, unit, code)
end

function PitBull_Name:OnClearUnitFrame(unit, frame)
	frame.nameText = delFrame(frame.nameText)
end

function PitBull_Name:OnUnknownLayout(unit, frame, name)
	if frame.healthBar then
		frame.nameText:SetPoint("LEFT", frame.healthBar, "LEFT", 3, 0)
		if frame.healthText then
			frame.nameText:SetPoint("RIGHT", frame.healthText, "LEFT", -5, 0)
		end
	end
end

local function getTextStyle(group)
	return PitBull_Name.db.profile[group].textStyle
end
local function setTextStyle(group, value)
	PitBull_Name.db.profile[group].textStyle = value
	
	local code
	if value == L["Custom"] then
		code = PitBull_Name.db.profile[group].textCustom
	else
		code = textStyles[value]
	end
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:RegisterWatchdogFontString(frame.nameText, unit, code)
	end
end
local function getTextCustom(group)
	return PitBull_Name.db.profile[group].textCustom
end
local function setTextCustom(group, value)
	PitBull_Name.db.profile[group].textCustom = value
	
	if PitBull_Name.db.profile[group].textStyle ~= L["Custom"] then
		return
	end
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull:RegisterWatchdogFontString(frame.nameText, unit, value)
	end
end

PitBull_Name:RegisterPitBullOptionsMethod(function(group)
	local textValidate = {}
	for k in pairs(textStyles) do
		textValidate[k] = k
	end
	return {
		type = 'group',
		name = L["Name Text"],
		desc = L["Name Text"],
		args = {
			style = {
				type = 'text',
				name = L["Style"],
				desc = L["Style"],
				validate = textValidate,
				get = getTextStyle,
				set = setTextStyle,
				passValue = group,
			},
			custom = {
				type = 'text',
				name = L["Custom"],
				desc = L["Custom"],
				get = getTextCustom,
				set = setTextCustom,
				usage = L["Watchdog tags"],
				passValue = group,
			}
		}
	}
end)


wrap(PitBull_Name, 'Name')
