local PitBull = PitBull
local PitBull_Portrait = PitBull:NewModule("Portrait", "AceEvent-2.0")
local self = PitBull_Portrait

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame
local wrap = PitBull.wrap

PitBull_Portrait:RegisterPitBullChildFrames('portrait')

function PitBull_Portrait:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Portrait")
	PitBull:RegisterDefaults("Portrait", "profile", {
		['**'] = {
			hidden = false,
			style = '3D',
		},
		targettarget = {
			hidden = true,
		},
		targettargettarget = {
			hidden = true,
		},
	})
end

function PitBull_Portrait:OnEnable()
	self:RegisterEvent("UNIT_PORTRAIT_UPDATE")
end

function PitBull_Portrait:UNIT_PORTRAIT_UPDATE(unit)
	local frame = PitBull:GetUnitFrame(unit)
	if frame then
		self:Update(unit, frame)
	end
end

local classIcons = {
	["WARRIOR"] = {0, 0.25, 0, 0.25},
	["MAGE"] = {0.25, 0.49609375, 0, 0.25},
	["ROGUE"] = {0.49609375, 0.7421875, 0, 0.25},
	["DRUID"] = {0.7421875, 0.98828125, 0, 0.25},
	["HUNTER"] = {0, 0.25, 0.25, 0.5},
	["SHAMAN"] = {0.25, 0.49609375, 0.25, 0.5},
	["PRIEST"] = {0.49609375, 0.7421875, 0.25, 0.5},
	["WARLOCK"] = {0.7421875, 0.98828125, 0.25, 0.5},
	["PALADIN"] = {0, 0.25, 0.5, 0.75},
}
function PitBull_Portrait:Update(unit, frame)
	if frame.portrait then
		local style = PitBull_Portrait.db.profile[frame.group].style
		if style == '3D' then
			if not UnitExists(unit) or not UnitIsConnected(unit) or not UnitIsVisible(unit) then
				local name = "question"
				if name == frame.portrait.lastName and PitBull.IsWackyUnit[unit] then
					return
				end
				frame.portrait.lastName = name
				frame.portrait:SetModelScale(4.25)
				frame.portrait:SetPosition(0,0,-1.5)
				frame.portrait:SetModel("Interface\\Buttons\\talktomequestionmark.mdx")
			else
				local name = UnitName(unit)
				if name == frame.portrait.lastName and PitBull.IsWackyUnit[unit] then
					return
				end
				frame.portrait.lastName = name
				frame.portrait:SetUnit(unit)
				frame.portrait:SetCamera(0)
				frame.portrait:Show()
			end
		elseif style == '2D' then
			SetPortraitTexture(frame.portrait, unit)
		elseif style == 'Class' then
			if UnitIsPlayer(unit) then
				--"SetPortraitToTexture('texture', icon)" useful here?  Investigate.
				--Also, I'm doing this how Blizzard does it in their raidframes.  It's drycoded, so some of these steps might be unnecessary.
				local class = classIcons[select(2,UnitClass(unit))]
				frame.portrait:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
				frame.portrait:SetTexCoord(class[1], class[2], class[3], class[4])
			else
				--Unit is a mob, and the class icon would this be either WARRIOR or PALADIN, which isn't very helpful.
				--For now, throw it to 2d.  Hunt down in-game icons for the output of UnitClassification, maybe?
				frame.portrait:SetTexCoord(0,1,0,1)
				SetPortraitTexture(frame.portrait, unit)
			end
		end
	end
end
--[[
function PitBull_Portrait:UpdateAllNonWacky()
	for unit, frame in PitBull:IterateNonWackyUnitFrames() do
		self:Update(unit, frame)
	end
end
]]
function PitBull_Portrait:OnUpdateFrame(unit, frame)
	self:Update(unit, frame)
end

function PitBull_Portrait:OnPopulateUnitFrame(unit, frame)
	if not PitBull_Portrait.db.profile[frame.group].hidden then
		local style = PitBull_Portrait.db.profile[frame.group].style
		local portrait
		if style == '3D' then
			portrait = newFrame("PlayerModel", frame)
		else
			portrait = newFrame("Texture", frame, "ARTWORK")
		end
		frame.portrait = portrait
		local portraitBG = newFrame("Texture", frame, "BACKGROUND")
		frame.portrait.bg = portraitBG
		portraitBG:SetTexture(0, 0, 0, 0.25)
		portraitBG:SetAllPoints(portrait)
	end
end

function PitBull_Portrait:OnClearUnitFrame(unit, frame)
	if frame.portrait then
		frame.portrait.bg = delFrame(frame.portrait.bg)
		frame.portrait.lastName = nil
		frame.portrait = delFrame(frame.portrait)
	end
end

function PitBull_Portrait:OnUpdateLayout(unit, frame)
	if frame.portrait then
		local size = frame:GetHeight() - 10
		local portraitSize = math.max(frame.portrait:GetWidth(), frame.portrait:GetHeight())
		if portraitSize < size and portraitSize ~= 0 then
			size = portraitSize
		end
		frame.portrait:SetWidth(size)
		frame.portrait:SetHeight(size)
		self:Update(unit,frame)
	end
end

function PitBull_Portrait:OnUnknownLayout(unit, frame)
	if unit == "target" then
		frame.portrait:SetPoint("LEFT", frame, "RIGHT", 2, 0)
	else
		frame.portrait:SetPoint("RIGHT", frame, "LEFT", -2, 0)
	end
end

local function getHide(group)
	return PitBull_Portrait.db.profile[group].hidden
end
local function setHide(group, value)
	PitBull_Portrait.db.profile[group].hidden = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		if value then
			self:OnClearUnitFrame(unit, frame)
		else
			self:OnPopulateUnitFrame(unit, frame)
			self:OnUpdateFrame(unit, frame)
		end
		PitBull:UpdateLayout(unit)
	end
end
local function getStyle(group)
	return PitBull_Portrait.db.profile[group].style
end
local function setStyle(group, value)
	if value ~= PitBull_Portrait.db.profile[group].style then
		PitBull_Portrait.db.profile[group].style = value
		
		for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
			self:OnClearUnitFrame(unit, frame)
			self:OnPopulateUnitFrame(unit, frame)
			self:OnUpdateFrame(unit, frame)
			
			PitBull:UpdateLayout(unit)
		end
	end
end
PitBull_Portrait:RegisterPitBullOptionsMethod(function(group)
--	if group == "player" or group == "target" or group == "targettarget" or group == "party" or group == "pet" or group == "partypet" or group == "raid" or group == "focus" or group == "mouseover" then
		return {
			name = "Portrait",
			desc = "Portrait options",
			type = 'group',
			args = {
				hide = {
					type = 'toggle',
					name = "Hide portrait",
					desc = "Hide portrait",
					get = getHide,
					set = setHide,
					passValue = group,
				},
				style = {
					type = 'text',
					name = "Style",
					desc = "Style",
					get = getStyle,
					set = setStyle,
					passValue = group,
					validate = {'2D', '3D', 'Class',},
				},
			}
		}
--	end
end)

wrap(PitBull_Portrait, 'Portrait')